// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_POSIX_SERVERSOCKET_H
#define HU_VARADIISTVAN_SCPL_IO_POSIX_SERVERSOCKET_H
//------------------------------------------------------------------------------

#include "Waitable.h"

#include <cstdlib>

//------------------------------------------------------------------------------

struct sockaddr;

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

class Acceptor;

//------------------------------------------------------------------------------

/**
 * A socket in a server role. It can only bind to a new address,
 * listen and accept connections. It cannot connect to other sockets
 * and cannot be read or written.
 */
class ServerSocket : public Waitable
{
protected:
    /**
     * Construct the server socket.
     */
    ServerSocket(int domain, int type, Waiter* waiter = 0, int protocol = 0);

public:
    /**
     * Destroy the server socket.
     */
    virtual ~ServerSocket();

protected:
    /**
     * Bind the socket to the given address.
     *
     * @param reuseaddr if set to true, the SO_REUSEADDR socket option
     * will be set.
     *
     * @return if the binding has succeeded. If it is false, the error
     * code will be set accordingly.
     */
    bool bind(const struct sockaddr* addr, size_t addrlen,
              bool reuseaddr = true);

    /**
     * Start listening on the socket.
     *
     * @return if the call has succeeded. If it is false, the error
     * code will be set accordingly.
     */
    bool listen(int backlog = 5);

    /**
     * Accept a socket.
     *
     * @return the accepted socket. If may be -1, if the socket is
     * non-blocking, and no connection is available yet, or if another
     * error occured, in which case the error code will be set properly.
     */
    int accept(struct sockaddr* addr, size_t* addrlen);

    friend class Acceptor;
};

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_POSIX_SERVERSOCKET_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
